@@ -23,6 +23,8 @@
 (require 'compile)
 (require 'info-look)
 
+(eval-when-compile (require 'cl))
+
 
 ;;; Customization:
 
@@ -114,7 +116,8 @@ effect on new REPLs. For existing ones, use the command
   :type 'boolean
   :group 'geiser-guile)
 
-(geiser-custom--defcustom geiser-guile-manual-lookup-nodes '("Guile" "guile-2.0")
+(geiser-custom--defcustom geiser-guile-manual-lookup-nodes
+                          '("Guile" "guile-2.0")
   "List of info nodes that, when present, are used for manual lookups"
   :type '(repeat string)
   :group 'geiser-guile)
@@ -252,6 +255,7 @@ This function uses `geiser-guile-init-file' if it exists."
 (geiser-syntax--scheme-indent
  (c-declare 0)
  (c-lambda 2)
+ (lambda* 1)
  (pmatch defun)
  (sigaction 1)
  (with-fluid* 1)
@@ -285,6 +289,11 @@ This function uses `geiser-guile-init-file' if it exists."
 
 ;;; REPL startup
 
+(defconst geiser-guile-minimum-version "2.0")
+
+(defun geiser-guile--version (binary)
+  (shell-command-to-string (format "%s  -c '(display (version))'" binary)))
+
 (defun geiser-guile-update-warning-level ()
   "Update the warning level used by the REPL.
 The new level is set using the value of `geiser-guile-warning-level'."
@@ -301,7 +310,7 @@ it spawn a server thread."
   (interactive)
   (geiser-connect 'guile))
 
-(defun geiser-guile--set-load-path ()
+(defun geiser-guile--set-geiser-load-path ()
   (let* ((path (expand-file-name "guile/" geiser-scheme-dir))
          (witness "geiser/emacs.scm")
          (code `(begin (if (not (%search-load-path ,witness))
@@ -314,12 +323,14 @@ it spawn a server thread."
        `((,geiser-guile--path-rx geiser-guile--resolve-file-x)
          ("^  +\\([0-9]+\\):\\([0-9]+\\)" nil 1 2)))
   (compilation-setup t)
-  (font-lock-add-keywords nil
-                          `((,geiser-guile--path-rx 1
-                                                    compilation-error-face)))
+  (font-lock-add-keywords nil `((,geiser-guile--path-rx
+                                 1 compilation-error-face)))
   (let ((geiser-log-verbose-p t))
-    (when remote (geiser-guile--set-load-path))
+    (when remote (geiser-guile--set-geiser-load-path))
     (geiser-eval--send/wait ",use (geiser emacs)\n'done")
+    (dolist (dir geiser-guile-load-path)
+      (let ((dir (expand-file-name dir)))
+        (geiser-eval--send/wait `(:eval (:ge add-to-load-path ,dir)))))
     (geiser-guile-update-warning-level)))
 
 
@@ -358,6 +369,8 @@ it spawn a server thread."
 (define-geiser-implementation guile
   (binary geiser-guile--binary)
   (arglist geiser-guile--parameters)
+  (version-command geiser-guile--version)
+  (minimum-version geiser-guile-minimum-version)
   (repl-startup geiser-guile--startup)
   (prompt-regexp geiser-guile--prompt-regexp)
   (debugger-prompt-regexp geiser-guile--debugger-prompt-regexp)
