@@ -20,6 +20,8 @@
 (require 'geiser-base)
 (require 'geiser)
 
+(eval-when-compile (require 'cl))
+
 
 ;;; Customization:
 
@@ -113,18 +115,47 @@ using start-geiser, a procedure in the geiser/server module."
 
 ;;; Evaluation support:
 
+(defconst geiser-racket--module-re
+  "^(module[+*]? +\\([^ ]+\\)\\W+\\([^ ]+\\)?")
+
+(defun geiser-racket--explicit-module ()
+  (save-excursion
+    (ignore-errors
+      (while (not (zerop (geiser-syntax--nesting-level)))
+        (backward-up-list)))
+    (and (looking-at geiser-racket--module-re)
+         (let ((mod (match-string-no-properties 1))
+               (lang (match-string-no-properties 2)))
+           (cons (geiser-syntax--form-from-string mod)
+                 (geiser-syntax--form-from-string lang))))))
+
 (defun geiser-racket--language ()
+  (or (cdr (geiser-racket--explicit-module))
+      (save-excursion
+        (goto-char (point-min))
+        (if (re-search-forward "^#lang +\\([^ ]+\\)" nil t)
+            (geiser-syntax--form-from-string (match-string-no-properties 1))))
+      "#f"))
+
+(defun geiser-racket--implicit-module ()
   (save-excursion
     (goto-char (point-min))
-    (if (re-search-forward
-         "^\\(?:#lang\\|(module +[^ ]+?\\) +\\([^ ]+?\\|([^)]+)\\) *$" nil t)
-        (car (geiser-syntax--read-from-string (match-string-no-properties 1)))
-      "#f")))
+    (when (re-search-forward "^#lang " nil t)
+      (buffer-file-name))))
+
+(defun geiser-racket--find-module ()
+  (let ((bf (geiser-racket--implicit-module))
+        (sub (car (geiser-racket--explicit-module))))
+    (cond ((and (not bf) (not sub)) nil)
+          ((and (not bf) sub) sub)
+          (sub `(submod (file ,bf) ,sub))
+          (t bf))))
 
 (defun geiser-racket--enter-command (module)
-  (when (stringp module)
+  (when (or (stringp module) (listp module))
     (cond ((zerop (length module)) ",enter #f")
-          ((file-name-absolute-p module) (format ",enter %S" module))
+          ((or (listp module)
+               (file-name-absolute-p module)) (format ",enter %S" module))
           (t (format ",enter %s" module)))))
 
 (defun geiser-racket--geiser-procedure (proc &rest args)
@@ -135,33 +166,12 @@ using start-geiser, a procedure in the geiser/server module."
              (geiser-racket--language)
              (mapconcat 'identity (cdr args) " ")))
     ((load-file compile-file)
-     (format ",geiser-eval geiser/main racket (geiser:%s %s)"
-             proc (car args)))
+     (format ",geiser-load %S" (geiser-racket--find-module)))
     ((no-values) ",geiser-no-values")
     (t (format ",apply geiser:%s (%s)" proc (mapconcat 'identity args " ")))))
 
-(defconst geiser-racket--module-re
-  "^(module +\\([^ ]+\\)")
-
-(defun geiser-racket--explicit-module ()
-  (save-excursion
-    (goto-char (point-min))
-    (and (re-search-forward geiser-racket--module-re nil t)
-         (ignore-errors
-           (car (geiser-syntax--read-from-string
-                 (match-string-no-properties 1)))))))
-
-(defsubst geiser-racket--implicit-module ()
-  (save-excursion
-    (goto-char (point-min))
-    (if (re-search-forward "^#lang " nil t)
-        (buffer-file-name)
-      :f)))
-
 (defun geiser-racket--get-module (&optional module)
-  (cond ((and (null module) (buffer-file-name)))
-        ;; (geiser-racket--explicit-module)
-        ((null module) (geiser-racket--implicit-module))
+  (cond ((null module) (or (geiser-racket--find-module) :f))
         ((symbolp module) module)
         ((and (stringp module) (file-name-absolute-p module)) module)
         ((stringp module) (make-symbol module))
@@ -253,9 +263,15 @@ using start-geiser, a procedure in the geiser/server module."
 
 ;;; Keywords and syntax
 
+(setq geiser-racket-font-lock-forms
+  '(("^#lang\\>" . 0)
+    ("\\[\\(else\\)\\>" . 1)
+    ("(\\(define/match\\)\\W+[[(]?\\(\\w+\\)+\\b"
+     (1 font-lock-keyword-face)
+     (2 font-lock-function-name-face))))
+
 (defun geiser-racket--keywords ()
-  (append '(("^#lang\\>" . 0)
-            ("\\[\\(else\\)\\>" . 1))
+  (append geiser-racket-font-lock-forms
           (when geiser-racket-extra-keywords
             `((,(format "[[(]%s\\>" (regexp-opt geiser-racket-extra-keywords 1))
                . 1)))))
@@ -313,10 +329,12 @@ using start-geiser, a procedure in the geiser/server module."
  (letrec: 1)
  (local 1)
  (match-let 1)
+ (match-let-values 1)
  (match/values 1)
  (mixin 2)
  (module defun)
  (module+ defun)
+ (module* defun)
  (parameterize-break 1)
  (quasisyntax/loc 1)
  (send* 1)
@@ -340,7 +358,12 @@ using start-geiser, a procedure in the geiser/server module."
  (with-handlers: 1))
 
 
-;;; Startup
+;;; REPL Startup
+
+(defvar geiser-racket-minimum-version "5.3")
+
+(defun geiser-racket--version (binary)
+  (shell-command-to-string (format "%s  -e '(display (version))'" binary)))
 
 (defun geiser-racket--startup (remote)
   (set (make-local-variable 'compilation-error-regexp-alist)
@@ -352,6 +375,41 @@ using start-geiser, a procedure in the geiser/server module."
     (setq geiser-image-cache-dir
           (geiser-eval--send/result '(:eval (image-cache) geiser/user)))))
 
+
+;;; Additional commands
+
+(defvar geiser-racket--submodule-history ())
+
+(defun geiser-racket--submodule-form (name)
+  (format "module[+*]? %s"
+          (cond ((eq 1 name) "")
+                ((numberp name)
+                 (read-string "Submodule name: " nil
+                              'geiser-racket--submodule-history))
+                ((stringp name) name)
+                (t ""))))
+
+(defun geiser-racket-toggle-submodules (&optional name)
+  "Toggle visibility of submodule forms.
+
+Use a prefix to be asked for a submodule name."
+  (interactive "p")
+  (geiser-edit--toggle-visibility (geiser-racket--submodule-form name)))
+
+(defun geiser-racket-show-submodules (&optional name)
+  "Unconditionally shows all submodule forms.
+
+Use a prefix to be asked for a submodule name."
+  (interactive "p")
+  (cond ((eq 1 name) (geiser-edit--show-all))
+        (t (geiser-edit--show (geiser-racket--submodule-form name)))))
+
+(defun geiser-racket-hide-submodules (&optional name)
+  "Unconditionally hides all visible submodules.
+
+Use a prefix to be asked for a submodule name."
+  (interactive "p")
+  (geiser-edit--hide (geiser-racket--submodule-form name)))
 
 
 ;;; Implementation definition:
@@ -359,6 +417,8 @@ using start-geiser, a procedure in the geiser/server module."
 (define-geiser-implementation racket
   (unsupported-procedures '(callers callees generic-methods))
   (binary geiser-racket--binary)
+  (minimum-version geiser-racket-minimum-version)
+  (version-command geiser-racket--version)
   (arglist geiser-racket--parameters)
   (repl-startup geiser-racket--startup)
   (prompt-regexp geiser-racket--prompt-regexp)
