@@ -197,11 +197,15 @@ implementation-specific entries for font-lock-keywords.")
   (when (stringp string)
     (let* ((start (or start 0))
            (end (or end (length string)))
-           (max-lisp-eval-depth (max max-lisp-eval-depth (- end start))))
+           (max-lisp-eval-depth (min 20000
+                                     (max max-lisp-eval-depth (- end start)))))
       (with-temp-buffer
         (save-excursion (insert string))
         (cons (ignore-errors (geiser-syntax--read)) (point))))))
 
+(defun geiser-syntax--form-from-string (s)
+  (car (geiser-syntax--read-from-string s)))
+
 (defsubst geiser-syntax--form-after-point (&optional boundary)
   (let ((geiser-syntax--read/buffer-limit (and (numberp boundary) boundary)))
     (save-excursion (values (geiser-syntax--read) (point)))))
